package LD28 
{
	import mx.collections.ArrayCollection;
	import mx.containers.dividedBoxClasses.BoxDivider;
	import mx.containers.utilityClasses.Flex;
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Vanja Radenovic
	 */
	public class HallState extends FlxState 
	{
		
		private var background:FlxSprite;
		public var legislators:Vector.<Legislator> = new Vector.<Legislator>();
		
		public var chosenLegislatorNames:ArrayCollection = new ArrayCollection();
		private var scenarioXML:XML;
		
		private var legislatorFirstRowY:int = 16;
		private var legislatorSecondRowY:int = 160;
		private var legislatorSpacingX:int = 105;
		
		private var secondsDisplay:FlxText;
		
		private var instructionCard:FlxSprite;
		private var instructionSeen:Boolean;
		
		private var successGrp:FlxGroup;
		private var successOverlay:FlxSprite;
		private var successBtn:FlxButton;
		private var success:Boolean;
		
		private var endBackground:FlxSprite;
		private var endView:Boolean;
		
		override public function create():void 
		{
			ID = 1;
			FlxG.play(Content.ClickSnd);
			
			// instanciate a new words in play collection
			TenSecondsGame.wordsInPlay = new ArrayCollection();
						
			background = new FlxSprite(0, 0, Content.BackgroundImg);
			add(background);
			
			// hand the xml
			// load xml file (load into a string first then into the XML object or it will not work)
			var xmlString:String = new Content.ScenariosXML;
			scenarioXML = new XML(xmlString);
			
			
			var i:int = 0;
			
			// get the list of all the legislators in our scenairo
			for each (var legislatorXML:XML in scenarioXML.Scenario.(@ID == TenSecondsGame.scenario).Legislator)
			{	
				var legislator:Legislator;
				
				// select a legsilator name at random ensuring its not already taken
				var isTaken:Boolean = true;
				
				while (isTaken)
				{
					var name:String = TenSecondsGame.legislatorNames[(FlxG.random() * TenSecondsGame.legislatorNames.length)];
					isTaken = chosenLegislatorNames.contains(name);
				}
				chosenLegislatorNames.addItem(name);
				
				if (i < 6)
				{
					legislator = new Legislator(28 + (i * legislatorSpacingX), legislatorFirstRowY, name);
				}
				else
				{
					legislator = new Legislator(28 + ((i - 6) * legislatorSpacingX), legislatorSecondRowY, name);
				}
				
				
				
				legislators.push(legislator);
				
				i++;				
			}
			
			// now do the preferences
			
			i = 0;
			
			// get the list of all the legislators in our scenairo
			for each (legislatorXML in scenarioXML.Scenario.(@ID == TenSecondsGame.scenario).Legislator)
			{	
				for each (var preferenceXML:XML in legislatorXML.Preference)
				{
					legislators[i].addPreference(preferenceXML.ID, preferenceXML.Type, preferenceXML.Value);
				}
				i++;
			}
			
						
			// the text at the bottom
			new SpeechField(new WordChoicesUI(this), scenarioXML);
			
			// the seconds counter at the top
			secondsDisplay = new FlxText(18, 0, 70, "0 Seconds");
			secondsDisplay.setFormat("MainFont", 10, 0xff000000, "center");
			//add(secondsDisplay);
			
			// add the scenario introduction card
			switch (TenSecondsGame.scenario)
			{
				case 1:
					instructionCard = new FlxSprite(0, 0, Content.IntroCard1Img);
					break;
				case 2:
					instructionCard = new FlxSprite(0, 0, Content.IntroCard2Img);
					break;
				case 3:
					instructionCard = new FlxSprite(0, 0, Content.IntroCard3Img);
					break;
				case 4:
					instructionCard = new FlxSprite(0, 0, Content.IntroCard4Img);
					break;
				case 5:
					instructionCard = new FlxSprite(0, 0, Content.IntroCard5Img);
					break;
				case 6:
					instructionCard = new FlxSprite(0, 0, Content.IntroCard6Img);
					break;
				default: // so compiler is happy
					instructionCard = new FlxSprite(0, 0, Content.IntroCard1Img);
					break;
				
			}
			add(instructionCard);
			// tell flixel we want to control the mouse
			FlxG.onMouseUp = mouseClick;
			
			endBackground = new FlxSprite(0, 0, Content.EndBackgroundImg);
			endBackground.visible = false;
			add(endBackground);
			
			
			// instanciate the success group
			successGrp = new FlxGroup();
			add(successGrp);
		}
		
		
		public function updateLegislators():void
		{
			for each (var legislator:Legislator in legislators)
			{
				legislator.testWords();
			}
			
			for each (legislator in legislators)
			{
				legislator.testLegislators();
			}
			// reset seconds
			TenSecondsGame.seconds = 0;
			// count seconds
			for each (legislator in legislators)
			{
				TenSecondsGame.seconds += legislator.updateFinal();
				
			}
			
			secondsDisplay.text = TenSecondsGame.seconds + " seconds";
			
			if (TenSecondsGame.seconds >= 10)
			{
				// success time
				
				FlxG.play(Content.ClappingSnd);
				successOverlay = new FlxSprite(204, 85, Content.SuccessOverlayImg);
				successGrp.add(successOverlay);
				
				successBtn = new FlxButton(259, 136, "Forward!", successBtnClick, 120);
				successBtn.loadGraphic(Content.WordButtonImg);
				successBtn.label.setFormat("MainFont", 14, 0xff000000, "center");
				successGrp.add(successBtn);
								
				
			}
			
		}
		
		public function successBtnClick():void
		{
			if (TenSecondsGame.scenario == 6) // victory
			{
				endBackground.visible = true;
				FlxG.onMouseUp = mouseClick;
				endView = true;
				success = false;
				successGrp.visible = false;
			}
			else // next scenario
			{
				FlxG.play(Content.ClickSnd);
				TenSecondsGame.scenario++;
				FlxG.switchState(new HallState);
			}
			
		}
		
		public function mouseClick():void
		{
			if (!instructionSeen)
			{
				FlxG.play(Content.ClickSnd);
				instructionCard.visible = false;
				FlxG.onMouseUp = null;
				instructionSeen = true;
			}
			
			if (endView)
			{
				
				FlxG.onMouseUp = null;
				FlxG.switchState(new MainState);
			}
			
		}
		
		override public function update():void
		{
			// add support for enter and space
			if (FlxG.keys.justPressed("ENTER") || FlxG.keys.justPressed("SPACE"))
			{
				mouseClick();
			}
			
			if (!success)
			{
				super.update();
			}
			else
			{
				successGrp.update();
			}
		
		}
		
	}

}